package com.malk.service.dingtalk;

import java.io.File;
import java.util.List;
import java.util.Map;

/**
 * OA审批
 *
 * @apiNote https://open.dingtalk.com/document/orgapp/workflow-overview
 */
public interface DDClient_Workflow {

    /**
     * 发起审批实例 [旧版本有字段报错提示]
     *
     * @param body_ext 包含必填字段, 不能为空, 统一扩展合并到请求参数内
     * @implNote 若approvers已传值时（即直接指定审批人列表, 最大列表长度：20。），则deptId不需填写。若approvers未传值时（即不直接指定审批人列表），则deptId需必填，若为根部门ID需填1。
     * @implNote targetSelectActioners 使用OA后台设置的默认流程，并且流程中有审批人自选节点，该参数必填。
     * @implNote formComponentValues 表单数据内容，控件列表，最大列表长度：150。
     * @apiNote https://open.dingtalk.com/document/orgapp-service/initiate-approval
     */
    String doProcessInstances(String access_token, String originatorUserId, String processCode, List formComponentValues, Map body_ext);

    /**
     * 获取审批实例详情
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/obtains-the-details-of-a-single-approval-instance
     */
    Map getProcessInstanceId(String access_token, String processInstanceId);

    /**
     * 撤回审批实例 NOTE: 终审通过无法撤销，若是假勤管理单据才能通知撤销
     *
     * @param processInstanceId 审批实例ID
     * @param isSystem          是否通过系统操作
     * @param remark            终止说明
     * @param operatingUserId   操作人的userid, 当isSystem为false时，该参数必传
     * @apiNote https://open.dingtalk.com/document/orgapp-service/terminate-a-workflow-by-using-an-instance-id
     */
    boolean terminateRunningApprove(String access_token, String processInstanceId, boolean isSystem, String remark, String operatingUserId);

    /**
     * 通知审批撤销
     *
     * @apiNote https://open.dingtalk.com/document/orgapp/notify-the-attendance-to-modify-the-punch-result-when-the
     */
    boolean cancelRunningApprove(String access_token, String userid, String approve_id);

    /**
     * 添加审批评论
     *
     * @param processInstanceId 审批实例ID
     * @param file              文件
     * @param text              评论的内容
     * @param commentUserid     评论人的userid
     * @apiNote https://open.dingtalk.com/document/orgapp-service/add-an-approval-comment
     */
    boolean commentApproveInstance(String access_token, String processInstanceId, File file, String text, String commentUserid);

    /**
     * 获取审批实例ID列表
     *
     * @param body_ext 扩展参数
     *                 - nextToken 分页游标: 如果是首次调用，该参数不传。
     *                 - maxResults 分页参数，每页大小，最多传20。
     *                 - userIds 发起人userId列表，最大列表长度为10。
     *                 - statuses 流程实例状态：NEW：新创建, RUNNING：审批中, TERMINATED：被终止, COMPLETED：完成, CANCELED：取消
     * @apiNote https://open.dingtalk.com/document/orgapp/obtain-an-approval-list-of-instance-ids
     */
    Map getInstanceIds(String access_token, String processCode, long startTime, long endTime, Map body_ext);
}
