package com.malk.service.dingtalk.impl;

import cn.hutool.core.util.ObjectUtil;
import com.malk.Util.UtilHttp;
import com.malk.Util.UtilMap;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.dingtalk.DDClient_Workflow;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.io.File;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class DDClientImpl_Workflow implements DDClient_Workflow {

    /**
     * 发起审批实例
     */
    @Override
    public String doProcessInstances(String access_token, String originatorUserId, String processCode, List formComponentValues, Map body_ext) {
        Map header = DDConf.initTokenHeader(access_token);
        Map body = UtilMap.map("originator_user_id, process_code, form_component_values", originatorUserId, processCode, formComponentValues);
        body.putAll(body_ext);
        DDR ddr = DDR.doPost("https://oapi.dingtalk.com/topapi/processinstance/create", header, DDConf.initTokenParams(access_token), body);
        return ddr.getProcessInstanceId();
    }

    /**
     * 获取单个审批实例详情
     */
    @Override
    public Map getProcessInstanceId(String access_token, String processInstanceId) {
        Map header = DDConf.initTokenHeader(access_token);
        Map body = UtilMap.map("processInstanceId", processInstanceId);
        DDR_New ddr = DDR_New.doGet("https://api.dingtalk.com/v1.0/workflow/processInstances", header, body);
        return (Map) ddr.getResult();
    }

    /**
     * 撤销审批实例
     */
    @Override
    public boolean terminateRunningApprove(String access_token, String processInstanceId, boolean isSystem, String remark, String operatingUserId) {
        Map header = DDConf.initTokenParams(access_token);
        Map request = UtilMap.map("process_instance_id, is_system, remark, operating_userid", processInstanceId, isSystem, remark, operatingUserId);
        Map body = UtilMap.map("request", request);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/process/instance/terminate", null, header, body, DDR.class);
        return (Boolean) r.getResult();
    }

    /**
     * 通知审批撤销
     **/
    @Override
    public boolean cancelRunningApprove(String access_token, String userid, String approve_id) {
        Map header = DDConf.initTokenParams(access_token);
        Map body = UtilMap.map("approve_id, userid", approve_id, userid);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/approve/cancel", null, header, body, DDR.class);
        return (Boolean) r.getResult();
    }

    /**
     * 添加审批评论
     */
    @Override
    public boolean commentApproveInstance(String access_token, String processInstanceId, File file, String text, String commentUserid) {
        Map header = DDConf.initTokenParams(access_token);
        Map body = UtilMap.map("process_instance_id, file, text, comment_userid", processInstanceId, file, text, commentUserid);
        Map req = UtilMap.map("request", body);
        DDR r = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/process/instance/comment/add", header, req, DDR.class);
        return (Boolean) r.getResult();
    }

    /**
     * 获取审批实例ID列表
     */
    @Override
    public Map getInstanceIds(String access_token, String processCode, long startTime, long endTime, Map body_ext) {
        Map header = DDConf.initTokenHeader(access_token);
        Map body = UtilMap.map("processCode, startTime, endTime", processCode, startTime, endTime);
        if (ObjectUtil.isNotNull(body_ext)) {
            body.putAll(body_ext);
        }
        if (!body.containsKey("maxResults")) {
            body.put("maxResults", 20);
        }
        if (!body.containsKey("nextToken")) {
            body.put("nextToken", 0);
        }
        DDR_New ddr_new = DDR_New.doPost("https://api.dingtalk.com/v1.0/workflow/processes/instanceIds/query", header, null, body);
        return (Map) ddr_new.getResult();
    }
}
