package com.malk.service.fxiaoke.impl;

import cn.hutool.core.util.ObjectUtil;
import com.malk.Util.UtilMap;
import com.malk.Util.UtilToken;
import com.malk.server.fxiaoke.FXKConf;
import com.malk.server.fxiaoke.FXXR;
import com.malk.service.fxiaoke.FXKClient;
import lombok.Synchronized;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Pageable;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class FXKImplClient implements FXKClient {

    @Autowired
    private FXKConf fxkConf;

    private final static String UNIQUE_STORAGE_ID = "invalid-token-fxiaoke";

    /**
     * 获取访问授权
     *
     * @apiNote https://open.fxiaoke.com/wiki.html#artiId=17
     */
    @Synchronized
    @Override
    public String getAccessToken() {
        String accessToken = UtilToken.get(UNIQUE_STORAGE_ID);
        if (StringUtils.isNotBlank(accessToken)) return accessToken;
        Map body = new HashMap();
        body.put("appId", fxkConf.getAppId());
        body.put("appSecret", fxkConf.getAppSecret());
        body.put("permanentCode", fxkConf.getPermanentCode());
        FXXR r = FXXR.doPost("https://open.fxiaoke.com/cgi/corpAccessToken/get/V2", null, null, body);
        log.info("响应token, {}", r.getCorpAccessToken());
        accessToken = r.getCorpAccessToken();
        // token失效自动重置: FXK重新调用不会重置过期时间
        UtilToken.put(UNIQUE_STORAGE_ID, accessToken, r.getExpiresIn() * 1000L);
        return accessToken;
    }

    /**
     * 根据手机号查询员工
     *
     * @apiNote https://open.fxiaoke.com/wiki.html#artiId=1122
     */
    @Override
    public List<Map> getUserByMobile(String accessToken, String corpId, String mobile) {
        Map body = UtilMap.map("corpAccessToken, corpId, mobile", accessToken, corpId, mobile);
        FXXR fxxr = FXXR.doPost("https://open.fxiaoke.com/cgi/user/getByMobile", null, null, body);
        return fxxr.getEmpList();
    }

    /**
     * 获取考勤数据列表
     *
     * @apiNote https://open.fxiaoke.com/wiki.html#artiId=153
     */
    @Override
    public List<Map> getAttendanceForOutside(String accessToken, String corpId, long startTime, long endTime, List<String> openUserIds, Pageable pageable) {
        Map body = UtilMap.map("corpAccessToken, corpId, startTime, endTime, openUserIds", accessToken, corpId, startTime, endTime, openUserIds);
        if (ObjectUtil.isNull(pageable)) {
            body.put("pageSize", 1000);
            body.put("pageNumber", 1);
        }
        FXXR fxxr = FXXR.doPost("https://open.fxiaoke.com/cgi/attendance/find", null, null, body);
        return fxxr.getDatas();
    }
}
